*
*
************************************************************************
*
       Subroutine MODEL(IK,T0,P0,P0X,P0Y,P0Z,X0,Y0,Z0,IDE,IGEN,IDION) 
*      --------------------------------------------------------------
*
* IK  = 1 primary ion, 2,3,... =material component
* Example:
* dEdx(1,10002) is the full stopping power for the INITIAL particle +
*               composite material
* dEdx(2,10002) is the full stopping power for the FIRST component
*               of the material with the composite material  etc.
* dSdX1(1,1,10002) is the stopping power "considering defect production"
*               for the interaction of the INITIAL particle with 
*               the energy transfer to FIRST material component (this
*               PKA is moving in the media with the effective Z,A (see
*               separate code SL_LNSXX, XX=ver))
* dSdX1(1,2,10002) is for the INITIAL particle and energy
*               transfer to the SECOND material component
* dSdX1(2,1,10002) is for the FIRST material component and energy
*               transfer to the same FIRST material component
* dSdX1(2,2,10002) is for the FIRST material component and energy
*               transfer to the SECOND material component
* etc.
* NK            is the number of material components (max 7)
*
* IGEN          is the generation number (first PKA is 1)
* IDION         identifies the moving ion
*  
*
       parameter (maxc=21,maxc1=22)
       implicit real*8(A-H,O-Z)
       Common/BASIC/Z1,A1,Z2,A2,AW,RO,RN0  
       Common/Z2A2/ZT(MAXC),AT(MAXC),FR(MAXC),ED(MAXC),NK
       Common/Edmin/Edmin ! Edmin is not minimal displacement energy
       Common/STARS1/ dSdX1(MAXC1,MAXC,10002), 
     # dEdXNuc1(MAXC1,MAXC,10002), dEdXNuc0Ed1(MAXC1,MAXC,10002),
     # dEdXNucEdTm1(MAXC1,MAXC,10002), CST1(MAXC1,MAXC,10002), NMAX
       Common/SKY1/dEdx(MAXC1,10002),dEdxN1(MAXC1,10002),
     # Rp1(MAXC1,10002)
       Common/Interr/Interr1
       Common/SUN/Vel1L(405),Tmean1,Vel2Tm(405),Vel3Pl(405),
     #            R1,R1p,Tplay1,Vel4(405),Nel(405)
       Common/MOON/XX(105),DD(105),DDF(105),TK(105),HST
       Common/MEMORY/NFRENKEL(405)
       Common/SUN2/ZV5,Vel5,NN(405)
       Common/SATURN/TcritLAB(MAXC1)
       Common/WHATION/IDI
       Common/keytpka/keytpka
       Common/parpar/par_st1,par_st2
       Dimension CSS(MAXC)
*
* 
       T=T0
       Tcrit=TcritLAB(ik)
* general array index        
       JE=Idint((dlog10(T)+6.)*1000.)+1
                   If(JE.le.0.or.JE.gt.10002) Call error0(' JE ',11111)
*
*
*
         TplH1=0.0   ! Tplay for single history
*
  1      Continue      ! General loop
* Get "energy indicator number" for current T 
*   IDE corresponds only to the energy of primary particle 
*   and is not changed for any other generations
       Call INDICATOR(IK,T,IDE)
*
       If(IK.eq.1) NN(ide)=NN(ide)+1
*
*
* Estimation "elastic path" 
* general array index        
       JE=Idint((dlog10(T)+6.)*1000.)+1
                   If(JE.le.0.or.JE.gt.10002) Call error0(' JE ',11111)
* define total cross-section
       CSS(1)=FR(1)*CST1(IK,1,JE)
       If(NK.eq.1) goto 2
       DO MK=2,NK
       CSS(MK)=CSS(MK-1)+FR(MK)*CST1(IK,MK,JE)
       Enddo
 2     CSelT=CSS(NK)
                   If(CSelT.le.0.0) goto 2000
*
       dLel=1./(CSelT*RN0)  ! xsect in cm2
*
*
* STEP definition :
*
* Alternative  counting   DT0=dEdx(IK,JE)*STEP
! DT1 = only electronic component
       DT1=(   dEdx(IK,JE)-dEdxN1(IK,JE)  )
! DT2 = nucl stopp power for Tk < Ed
             dEdXNuc0Ed=0.0
             Do MK=1,NK 
             dEdXNuc0Ed=dEdXNuc0Ed+dEdXNuc0Ed1(IK,MK,JE)*FR(MK)
             Enddo 
       DT2=dEdXNuc0Ed
*
       DT3=DT1+DT2
* Free path/10.    ST1=dLel/10.
       ST1=par_st1*dLel
                                                      ST2=1.e+20
* Change of T = 0.05 due to el.loss and nucl.loss < Ed
*      If(DT3.gt.0.0) ST2=0.05*T/DT3
*
       If(DT3.gt.0.0) ST2=par_st2*T/DT3
       STEP=dmin1(ST1,ST2)
*
*
* Play if ELASTIC interaction occurs (for the transferring of the
* energy > Ed)
c
        PREL=STEP/dLel
        PR0=RANDOMg(0)
        IEL=0 
        COST=1.0
*
        if(PR0.le.PREL)               then
*"Elastic" interaction occurs !
        IEL=1
*
* Define (play) the type of the component of the composite material
        PR00=RANDOMg(0)
            If(NK.eq.1) then
            LK=1
            goto 3
            endif
        DO MK=1,NK 
        LK=MK
        If(PR00 .le. CSS(MK)/CSelt) goto 3
        ENDDO 
        print *,' It is not corr play:',CSS
  3     Continue
*
             Tmean=dEdXNucEdTm1(IK,LK,JE)/(CSelT*RN0)
        If(IK.eq.1 ) then 
             Nel(IDE)=Nel(IDE)+1
             DISPLL=dSdx1(IK,LK,JE)/(CSelT*RN0)
             Vel1L(IDE)=Vel1L(IDE)+DISPLL
ccc             Tmean=dEdXNucEdTm1(IK,LK,JE)/(CSelT*RN0)
             Tmean1=Tmean+Tmean1
             Vel2Tm(IDE)=Vel2Tm(IDE)+CF(LK+1,LK,Tmean)   ! LK
                      endif
* play T of PKA basing on dS/dT above Ed
        if(keytpka.lt.0) then
                         TPVA=  TPKAOLD(IK,LK,T,Tmean)                
                         else
                         TPVA=  TPKA(IK,LK,T,Tmean)                
                         endif
*
        if(IK.eq.1)  then
           Tplay1=Tplay1+TPVA
           Vel3Pl(IDE)=Vel3Pl(IDE)+CF(LK+1,LK,TPVA)     ! LK
                     endif
* 
******* If(T.le.Tcrit) goto 7000    !   MD
        Call smoothtransition(T,Tcrit,iweiter)
                                               if(iweiter==0) goto 7000
*
        CALL AFTER_INTERACTION(IK,LK,T,P0, P0X, P0Y, P0Z, TPVA,
     $                                 PVA,PVAX,PVAY,PVAZ)
*
        TplH1=TplH1+TPVA   ! Tplay for single history
*
*
******* If(TPVA.gt.Tcrit) then
        Call smoothtransition(TPVA,Tcrit,iweiter)
                                               if(iweiter==0) goto 6000

* identify new atom 
        IDI=IDI+1
*
        IGENPKA=IGEN+1
        Call WRITE99B(IK,LK,TPVA,X0,Y0,Z0,IGENPKA,IDION,IDI,T)
*
        CALL STORE_IN(LK,TPVA,PVA,PVAX,PVAY,PVAZ,X0,Y0,Z0,IDE,IGENPKA,
     @                IDI)
* number of Frenkel pairs
        NFRENKEL(IDE)=NFRENKEL(IDE)+1      
                                                              goto 6001
*******                   else  ! do not remember this particle
* MD
 6000                     Frcut=0.0  
        If(TPVA.gt.0.0)   FRcut=CF2(LK+1,LK,TPVA) +0.5
        NFRENKEL(IDE)=NFRENKEL(IDE)+Idint(FRcut)
******                    endif
 6001   continue 
*
********If(T.le.Tcrit) goto 7000    !   MD
        Call smoothtransition(T,Tcrit,iweiter)
                                               if(iweiter==0) goto 7000
*
*
        If(dsqrt(X0**2+Y0**2+Z0**2).le.ZV5) Vel5=Vel5+1.
*
        Do IR=1,105
        If(Z0.le.XX(IR)) then
        DDF(IR)=DDF(IR)+ 1.  ! Direct summing
        goto 499
        endif
        enddo
*
*
499     If(IK.ne.1) goto 510
        Do IR=1,105
        If(Z0.le.XX(IR)) then
        DD(IR)=DD(IR)+ DISPLL  ! Lindhard component is summing
        goto 500
        endif
        enddo
                                      endif !  if(PR0.le.PREL)
 500    continue
*End ---elastic
*
        Do IR=1,105
        If(Z0.le.XX(IR)) then
        TK(IR)=TK(IR)+ T*STEP
        goto 510
        endif
        enddo
*
510     continue
ccc        Write(777,*)(DT1+DT2)*STEP/T
*
******  If(T.le.Tcrit) goto 7000    !   MD
        Call smoothtransition(T,Tcrit,iweiter)
                                               if(iweiter==0) goto 7000
*
*
        T2=T-DT3*STEP
        If(T2.le.0.0) goto 550
        P2=dSqrt(T2/T)*P0
        P0X=P0X*P2/P0
        P0Y=P0Y*P2/P0
        P0Z=P0Z*P2/P0
        P0=P2
        T=T2
550     continue
*
*
******  If(T.le.Tcrit) goto 7000    !   MD
        Call smoothtransition(T,Tcrit,iweiter)
                                               if(iweiter==0) goto 7000
*
* stop option
*
cccccc      If(TplH1.gt.T00) then
cccccc      Interr1=Interr1+1
cccccc      goto 2000
cccccc      endif
*
* usual end of history
         If(T.lt.EDmin) goto 2000
         X0=X0+STEP*(P0X/P0)
         Y0=Y0+STEP*(P0Y/P0)
         Z0=Z0+STEP*(P0Z/P0)
*
*
*
         Goto 1
*
*
*
2000     continue
         Call WRITE99C(IK,IDION,T,X0,Y0,Z0,IGEN)
         If(IK.eq.1) then 
         R1=R1+dSqrt(X0**2+Y0**2+Z0**2)
         R1p=R1p+Z0  
                     endif
         Return
*
*
*
* Interrupt history taking MD results for the rest energy
7000    continue
*noalloy, because
        LK=1
                       Frcut=0.0  
cccc    If(T.gt.0.0)   FRcut=CF2(LK+1,LK,T) +0.5
        If(T.gt.0.0)   FRcut=CF2(IK,LK,T) +0.5
        NFRENKEL(IDE)= NFRENKEL(IDE)+idint(FRcut)
        goto 2000 
c
c
        End 
